
import { createRoot } from 'react-dom/client'
import App from './App.tsx'
import './index.css'

// Enhanced initialization function with better DOM readiness detection
const initializeReactApp = () => {
  // Configuration for retry attempts
  const MAX_RETRIES = 10;
  const INITIAL_DELAY = 50; // Start with a shorter initial delay
  const MAX_DELAY = 2000; // But don't wait too long between attempts
  
  // Function to find and mount the root element with exponential backoff
  const findRootAndMount = (retryCount = 0) => {
    try {
      // Log attempt for debugging
      console.log(`Attempting to find root element (attempt ${retryCount + 1}/${MAX_RETRIES})...`);
      
      // Try to find the root element
      const rootElement = document.getElementById("root");
      
      if (rootElement) {
        // Found the root element, check if it's ready for mounting
        if (document.readyState === 'complete' || document.readyState === 'interactive') {
          console.log("Root element found and document ready, mounting React application");
          const root = createRoot(rootElement);
          root.render(<App />);
          console.log("React application mounted successfully");
          return true;
        } else {
          // Document not fully ready yet, wait and try again
          console.log("Document not fully ready yet, waiting...");
          const delay = Math.min(INITIAL_DELAY * Math.pow(1.5, retryCount), MAX_DELAY);
          setTimeout(() => findRootAndMount(retryCount + 1), delay);
          return;
        }
      } 
      
      // Root not found, decide whether to retry
      if (retryCount < MAX_RETRIES) {
        const delay = Math.min(INITIAL_DELAY * Math.pow(1.5, retryCount), MAX_DELAY);
        console.log(`Root element not found. Retrying in ${delay}ms`);
        
        setTimeout(() => {
          findRootAndMount(retryCount + 1);
        }, delay);
      } else {
        // All retries failed, try emergency fallback
        console.error("Failed to find root element after maximum retries");
        
        // Create root element as last resort if it doesn't exist
        if (!document.getElementById("root")) {
          console.warn("Creating missing root element");
          const newRoot = document.createElement("div");
          newRoot.id = "root";
          document.body.appendChild(newRoot);
          
          // Try again with our new root
          setTimeout(() => {
            const lastChanceRoot = document.getElementById("root");
            if (lastChanceRoot) {
              console.log("Using created root element, mounting React application");
              createRoot(lastChanceRoot).render(<App />);
            }
          }, 100);
        }
      }
    } catch (error) {
      console.error("Error during React initialization:", error);
      
      // Try to recover from the error if possible
      if (retryCount < MAX_RETRIES) {
        const delay = Math.min(INITIAL_DELAY * Math.pow(2, retryCount), MAX_DELAY);
        console.log(`Error encountered. Retrying in ${delay}ms`);
        setTimeout(() => findRootAndMount(retryCount + 1), delay);
      }
    }
  };
  
  // Start the initialization process
  findRootAndMount();
};

// Check if the document is already ready
if (document.readyState === 'loading') {
  // Wait for the DOM to be ready before initializing
  document.addEventListener('DOMContentLoaded', () => {
    console.log("DOMContentLoaded fired, initializing React app");
    initializeReactApp();
  });
} else {
  // Document already loaded, initialize immediately
  console.log("Document already loaded, initializing React app immediately");
  initializeReactApp();
}

// Also add a backup initialization after window load
window.addEventListener('load', () => {
  console.log("Window load event fired");
  // Check if the root already has React mounted to avoid duplicate rendering
  const rootElement = document.getElementById("root");
  if (rootElement && rootElement.childNodes.length === 0) {
    console.log("Root element found with no children on window load, mounting React application");
    initializeReactApp();
  }
});

// Check if document body exists, and if not, wait for it
if (!document.body) {
  const bodyCheckInterval = setInterval(() => {
    if (document.body) {
      clearInterval(bodyCheckInterval);
      console.log("Document body now available");
      initializeReactApp();
    }
  }, 50);
}
